@@ -202,9 +202,9 @@ impl Array {
             )
         })
     }
-
-    pub fn save_rrfs(&self) -> Result<String> {
-        save_tensor(self.value.clone(), false).map(|_| self.tensor_hash_base16())
+    #[args(force = "false")]
+    pub fn save_rrfs(&self, force: bool) -> Result<String> {
+        save_tensor(self.value.clone(), force).map(|_| self.tensor_hash_base16())
     }
 
     pub fn tensor_hash_base16(&self) -> String {
