@@ -1,4 +1,4 @@
-use std::{collections::HashSet, fmt, iter::zip};
+use std::{fmt, iter::zip};
 
 use anyhow::{bail, Context, Result};
 use macro_rules_attribute::apply;
@@ -16,7 +16,7 @@ use rr_util::{
     python_error_exception, simple_from,
     tensor_util::{check_canon_idxs, Shape, TorchDeviceDtype},
 };
-use rustc_hash::FxHashMap as HashMap;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use thiserror::Error;
 use uuid::uuid;
 
