@@ -1,7 +1,7 @@
 #![feature(let_chains)]
 #![allow(clippy::borrow_deref_ref)]
 use std::{
-    collections::BTreeMap,
+    collections::{BTreeMap, BTreeSet},
     fmt::{self, Debug},
     hash::Hash,
     iter::zip,
@@ -21,6 +21,7 @@ use itertools::Itertools;
 use macro_rules_attribute::apply;
 pub use module::{Module, ModuleArgSpec, ModuleSpec};
 use num_bigint::BigUint;
+use opaque_iterative_matcher::OpaqueIterativeMatcherVal;
 use print::PrintOptions;
 use print_html::{PrintHtmlOptions, PrintOptionsBase};
 use pyo3::{
@@ -160,6 +161,10 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
     // FUNCTIONS BELOW HERE *shouldn't* be overridden by implementors!
     // (if you do implement, this won't be picked up on by CircuitNodeDefer (union types etc)!)
 
+    fn hash_base16(&self) -> String {
+        base16::encode_lower(&self.info().hash)
+    }
+
     fn init_info(self) -> Result<Self>
     where
         Self: Sized,
@@ -1903,6 +1908,8 @@ impl EqByBigHash for PyCircuitBase {
     }
 }
 
+const DEFAULT_FANCY_VALIDATE: bool = false;
+
 #[pymethods]
 impl PyCircuitBase {
     #[getter]
@@ -2161,6 +2168,57 @@ impl PyCircuitBase {
     pub fn into_var(&self) -> Option<Var> {
         self.0.clone().c().into()
     }
+    #[args(
+        cache_transform = "true",
+        cache_update = "true",
+        fancy_validate = "DEFAULT_FANCY_VALIDATE"
+    )]
+    pub fn update(
+        &self,
+        matcher: OpaqueIterativeMatcherVal,
+        transform: PyObject,
+        cache_transform: bool,
+        cache_update: bool,
+        fancy_validate: bool,
+    ) -> PyResult<CircuitRc> {
+        matcher.update(
+            self.0.clone(),
+            transform,
+            cache_transform,
+            cache_update,
+            fancy_validate,
+        )
+    }
+    #[args(fancy_validate = "DEFAULT_FANCY_VALIDATE")]
+    pub fn get(
+        &self,
+        matcher: OpaqueIterativeMatcherVal,
+        fancy_validate: bool,
+    ) -> PyResult<BTreeSet<CircuitRc>> {
+        matcher.get(self.0.clone(), fancy_validate)
+    }
+
+    #[args(fancy_validate = "DEFAULT_FANCY_VALIDATE")]
+    pub fn get_unique_op(
+        &self,
+        matcher: OpaqueIterativeMatcherVal,
+        fancy_validate: bool,
+    ) -> PyResult<Option<CircuitRc>> {
+        matcher.get_unique_op(self.0.clone(), fancy_validate)
+    }
+
+    #[args(fancy_validate = "DEFAULT_FANCY_VALIDATE")]
+    pub fn get_unique(
+        &self,
+        matcher: OpaqueIterativeMatcherVal,
+        fancy_validate: bool,
+    ) -> PyResult<CircuitRc> {
+        matcher.get_unique(self.0.clone(), fancy_validate)
+    }
+
+    pub fn are_any_found(&self, matcher: OpaqueIterativeMatcherVal) -> PyResult<bool> {
+        matcher.are_any_found(self.0.clone())
+    }
 }
 
 pub fn get_compatible_dtype(circ: &Circuit) -> TorchDeviceDtype {
