@@ -125,10 +125,8 @@ pub fn get_axis_leaves(
     }
     let circ_named_axes =
         propagate_named_axes(circuit, BTreeMap::from([(axis as u8, name.clone())]), true);
-    // circ_named_axes.print();
     let mut result_axis: HashMap<CircuitRc, usize> = Default::default();
     let mut result_no_axis: HashSet<CircuitRc> = Default::default();
-    let _names_to_remove = Some(HashSet::from_iter([name.clone()]));
     visit_circuit_with_parents_fallible(circ_named_axes.clone(), |x, parents| {
         // println!("child");
         // x.print();
