@@ -503,7 +503,7 @@ impl Parser {
                     "in parse, failed to convert ident={:?}, variant={}\nfull partial circuit: {:?}",
                     ident, ps.variant, ps
                 )
-            })?;
+            })?.rename(partial_circuits[ident].name.clone());
 
         context.insert(ident.clone(), result.clone());
         Ok(result)
