@@ -42,7 +42,7 @@ const ARROW: &'static str = "‣";
 const UP_ELBOW: &'static str = "└";
 const DOWN_ELBOW: &'static str = "┌";
 
-fn_struct!(pub CircuitToColorCode:Fn(circuit:CircuitRc)->usize);
+fn_struct!(pub CircuitToColorCode:Fn(circuit:CircuitRc)->Option<usize>);
 fn_struct!(pub CircuitCommenter:Fn(circuit:CircuitRc)->String);
 
 #[pyclass]
@@ -67,6 +67,10 @@ pub struct PrintOptions {
     pub comment_arg_names: bool,
     #[pyo3(get, set)]
     pub commenters: Vec<CircuitCommenter>,
+    #[pyo3(get, set)]
+    pub sync_tensors: bool,
+    #[pyo3(get, set)]
+    pub tensor_index_literal: bool,
 }
 
 impl Default for PrintOptions {
@@ -82,6 +86,8 @@ impl Default for PrintOptions {
             colorer: None,
             comment_arg_names: false,
             commenters: vec![],
+            sync_tensors: false,
+            tensor_index_literal: false,
         }
         .validate_ret()
         .unwrap()
@@ -181,7 +187,9 @@ impl PrintOptions {
         force_use_serial_numbers = "PrintOptions::default().force_use_serial_numbers",
         colorer = "None",
         comment_arg_names = "PrintOptions::default().comment_arg_names",
-        commenters = "PrintOptions::default().commenters"
+        commenters = "PrintOptions::default().commenters",
+        sync_tensors = "PrintOptions::default().sync_tensors",
+        tensor_index_literal = "PrintOptions::default().tensor_index_literal"
     )]
     pub fn new(
         bijection: bool,
@@ -194,6 +202,8 @@ impl PrintOptions {
         colorer: Option<CircuitToColorCode>,
         comment_arg_names: bool,
         commenters: Vec<CircuitCommenter>,
+        sync_tensors: bool,
+        tensor_index_literal: bool,
     ) -> Result<Self> {
         let result = Self {
             bijection,
@@ -206,6 +216,8 @@ impl PrintOptions {
             colorer,
             comment_arg_names,
             commenters,
+            sync_tensors,
+            tensor_index_literal,
         };
         result.validate()?;
         Ok(result)
@@ -221,7 +233,9 @@ impl PrintOptions {
         force_use_serial_numbers = "Default::default()",
         colorer = "Default::default()",
         comment_arg_names = "Default::default()",
-        commenters = "Default::default()"
+        commenters = "Default::default()",
+        sync_tensors = "Default::default()",
+        tensor_index_literal = "Default::default()"
     )]
     pub fn evolve(
         &self,
@@ -235,6 +249,8 @@ impl PrintOptions {
         colorer: MaybeNotSet<Option<CircuitToColorCode>>,
         comment_arg_names: MaybeNotSet<bool>,
         commenters: MaybeNotSet<Vec<CircuitCommenter>>,
+        sync_tensors: MaybeNotSet<bool>,
+        tensor_index_literal: MaybeNotSet<bool>,
     ) -> Result<Self> {
         let cloned = self.clone();
         Self::new(
@@ -254,6 +270,10 @@ impl PrintOptions {
             colorer.0.unwrap_or(cloned.colorer),
             comment_arg_names.0.unwrap_or(cloned.comment_arg_names),
             commenters.0.unwrap_or(cloned.commenters),
+            sync_tensors.0.unwrap_or(cloned.sync_tensors),
+            tensor_index_literal
+                .0
+                .unwrap_or(cloned.tensor_index_literal),
         )
     }
 
@@ -357,14 +377,14 @@ impl PrintOptions {
                 Circuit::Einsum(einsum) => einsum.get_spec().to_einsum_string(),
                 Circuit::Index(index) => {
                     if self.bijection {
-                        index.index.repr_bijection()?
+                        index.index.repr_bijection(self.tensor_index_literal)?
                     } else {
                         format!("{}", index.index)
                     }
                 }
                 Circuit::Scatter(scatter) => {
                     if self.bijection {
-                        scatter.index.repr_bijection()?
+                        scatter.index.repr_bijection(self.tensor_index_literal)?
                     } else {
                         format!("{}", scatter.index)
                     }
@@ -415,8 +435,11 @@ impl PrintOptions {
                     }
                 }
                 Circuit::Array(ac) => {
+                    if self.sync_tensors {
+                        ac.save_rrfs(true)?;
+                    }
                     if self.bijection {
-                        ac.save_rrfs()?;
+                        ac.save_rrfs(false)?;
                         ac.tensor_hash_base16()[..24].to_owned()
                     } else {
                         "".to_owned()
@@ -439,7 +462,10 @@ impl PrintOptions {
         });
         if let Some(threshold) = self.print_size_threshold {
             if circ.info().numel() > BigUint::from(threshold) && !circ.is_array() {
-                result.push_str(&format!(" # {}", color(&oom_fmt(circ.info().numel()), 0)));
+                result.push_str(&format!(
+                    " # {}",
+                    color(&oom_fmt(circ.info().numel()), Some(0))
+                ));
             }
         }
 
@@ -518,11 +544,15 @@ impl PrintOptions {
                 return Ok(());
             }
 
+            let mut used_color = false;
             if let Some(colorer) = &selfy.colorer {
-                result.push_str(&format!(
-                    "\u{001b}[{}m",
-                    COLOR_CODES[colorer.call(circ.clone().rc())? % COLOR_CODES.len()]
-                ));
+                if let Some(color) = colorer.call(circ.clone().rc())? {
+                    used_color = true;
+                    result.push_str(&format!(
+                        "\u{001b}[{}m",
+                        COLOR_CODES[color % COLOR_CODES.len()]
+                    ));
+                }
             }
 
             // allow_name_ident is_some iff !self.force_use_serial_numbers
@@ -563,7 +593,7 @@ impl PrintOptions {
                     .repr_line_info(circ.clone().rc())
                     .with_context(|| format!("repr line info failed for circuit={:?}", circ))?,
             );
-            if selfy.colorer.is_some() {
+            if used_color {
                 result.push_str("\u{001b}[0m");
             }
 
@@ -581,17 +611,7 @@ impl PrintOptions {
                     result.push('\n');
                     return Ok(());
                 };
-            for comment in node_comments.into_iter().chain(
-                selfy
-                    .commenters
-                    .iter()
-                    .map(|commenter| commenter.call(circ.clone().rc()))
-                    .collect::<Result<Vec<String>>>()?,
-            ) {
-                if !comment.is_empty() {
-                    write!(result, " # {}", comment).unwrap();
-                }
-            }
+            write_comment(result, node_comments, &selfy.commenters, circ.crc())?;
             result.push('\n');
             for (i, (child, new_traversal)) in circ.children().zip(new_traversal_per).enumerate() {
                 let new_last_child = is_last_child
@@ -600,31 +620,7 @@ impl PrintOptions {
                     .chain(std::iter::once(i == num_children - 1))
                     .collect();
                 let child_specific_commenters = if selfy.comment_arg_names {
-                    vec![match circ {
-                        Circuit::Einsum(ein) => ein.args[i]
-                            .1
-                            .iter()
-                            .map(|int| ALPHABET[*int as usize].clone())
-                            .join(""),
-                        Circuit::DiscreteVar(_dv) => match i {
-                            0 => "Values".to_owned(),
-                            1 => "Probs and Group".to_owned(),
-                            _ => {
-                                panic!()
-                            }
-                        },
-                        Circuit::Conv(_conv) => match i {
-                            0 => "Input".to_owned(),
-                            1 => "Filter".to_owned(),
-                            _ => {
-                                panic!()
-                            }
-                        },
-                        Circuit::StoredCumulantVar(scv) => {
-                            format!("K{}", scv.cumulants.keys().nth(i).unwrap())
-                        }
-                        _ => "".to_owned(),
-                    }]
+                    get_child_comments(circ, i)
                 } else {
                     vec![]
                 };
@@ -702,22 +698,22 @@ impl PrintOptions {
     }
 
     #[staticmethod]
-    pub fn variant_colorer() -> CircuitToColorCode {
+    pub fn type_colorer() -> CircuitToColorCode {
         CircuitToColorCode::new_dyn(Box::new(|circuit: CircuitRc| {
-            let mut hasher = std::collections::hash_map::DefaultHasher::default();
+            let mut hasher = rustc_hash::FxHasher::default();
             circuit.variant_string().hash(&mut hasher);
-            Ok(hasher.finish() as usize)
+            Ok(Some(hasher.finish() as usize))
         }))
     }
 
     #[staticmethod]
     pub fn hash_colorer() -> CircuitToColorCode {
         CircuitToColorCode::new_dyn(Box::new(|circuit: CircuitRc| {
-            Ok(circuit.info().hash_usize())
+            Ok(Some(circuit.info().hash_usize()))
         }))
     }
     #[staticmethod]
-    pub fn fixed_color(color: usize) -> CircuitToColorCode {
+    pub fn fixed_color(color: Option<usize>) -> CircuitToColorCode {
         CircuitToColorCode::new_dyn(Box::new(move |_circuit: CircuitRc| Ok(color)))
     }
 
@@ -744,28 +740,28 @@ fn init_print_options() -> PrintOptions {
     PrintOptions::debug_default(end_depth)
 }
 
-pub fn circuit_to_size_color_code(circuit: CircuitRc) -> usize {
+pub fn circuit_to_size_color_code(circuit: CircuitRc) -> Option<usize> {
     let size = circuit.info().numel_usize();
     if size == usize::MAX {
-        2
+        Some(2)
     } else if size > 10_000_000_000 {
-        3
+        Some(3)
     } else if size > 300_000_000 {
-        0
+        Some(0)
     } else if size > 300_000 {
-        1
+        Some(1)
     } else {
-        4
+        None
     }
 }
 
-pub fn circuit_to_computability_color_code(circuit: CircuitRc) -> usize {
-    if circuit.info().can_be_sampled {
-        1
+pub fn circuit_to_computability_color_code(circuit: CircuitRc) -> Option<usize> {
+    if !circuit.info().can_be_sampled {
+        Some(1)
     } else if !circuit.info().is_explicitly_computable {
-        2
+        Some(2)
     } else {
-        0
+        None
     }
 }
 
@@ -779,7 +775,7 @@ pub fn set_debug_print_options(options: PrintOptions) {
 
 pub fn debug_repr(circ: CircuitRc) -> Result<String> {
     Ok(format!(
-        "({})", // we wrap in parens to make it clear what's a single circuit.
+        "<{}>", /* we wrap in <> to make it clear what's a single circuit, similar to python <function __main__.<lambda>(x)> */
         DEBUG_PRINT_OPTIONS.lock().unwrap().repr(circ)?
     ))
 }
@@ -814,12 +810,15 @@ pub fn print_circuit_stats(circuit: &Circuit) {
     println!("{}", result);
 }
 
-pub fn color(string: &str, color_int: usize) -> String {
-    format!(
-        "\u{001b}[{}m{}\u{001b}[0m",
-        COLOR_CODES[color_int % COLOR_CODES.len()],
-        string
-    )
+pub fn color(string: &str, color_int: Option<usize>) -> String {
+    if let Some(color_int) = color_int {
+        return format!(
+            "\u{001b}[{}m{}\u{001b}[0m",
+            COLOR_CODES[color_int % COLOR_CODES.len()],
+            string
+        );
+    }
+    string.to_owned()
 }
 
 pub fn last_child_arrows(last_child: &Vec<bool>, is_output: bool, arrows: bool) -> String {
@@ -846,3 +845,50 @@ pub fn last_child_arrows(last_child: &Vec<bool>, is_output: bool, arrows: bool)
     }
     result
 }
+
+pub fn get_child_comments(parent: &Circuit, child_pos: usize) -> Vec<String> {
+    vec![match parent {
+        Circuit::Einsum(ein) => ein.args[child_pos]
+            .1
+            .iter()
+            .map(|int| ALPHABET[*int as usize].clone())
+            .join(""),
+        Circuit::DiscreteVar(_dv) => match child_pos {
+            0 => "Values".to_owned(),
+            1 => "Probs and Group".to_owned(),
+            _ => {
+                panic!()
+            }
+        },
+        Circuit::Conv(_conv) => match child_pos {
+            0 => "Input".to_owned(),
+            1 => "Filter".to_owned(),
+            _ => {
+                panic!()
+            }
+        },
+        Circuit::StoredCumulantVar(scv) => {
+            format!("K{}", scv.cumulants.keys().nth(child_pos).unwrap())
+        }
+        _ => "".to_owned(),
+    }]
+}
+
+pub fn write_comment(
+    result: &mut String,
+    node_comments: Vec<String>,
+    commenters: &Vec<CircuitCommenter>,
+    circuit: CircuitRc,
+) -> Result<()> {
+    for comment in node_comments.into_iter().chain(
+        commenters
+            .iter()
+            .map(|commenter| commenter.call(circuit.clone()))
+            .collect::<Result<Vec<String>>>()?,
+    ) {
+        if !comment.is_empty() {
+            write!(result, " # {}", comment).unwrap();
+        }
+    }
+    Ok(())
+}
