@@ -13,8 +13,8 @@ use crate::{
     opaque_iterative_matcher::OpaqueIterativeMatcherVal,
     prelude::*,
     print::{
-        circuit_to_computability_color_code, circuit_to_size_color_code, op_traversal_per,
-        PrintOptions,
+        circuit_to_computability_color_code, circuit_to_size_color_code, get_child_comments,
+        op_traversal_per, write_comment, CircuitCommenter, PrintOptions,
     },
     HashBytes,
 };
@@ -57,6 +57,10 @@ pub struct PrintHtmlOptions {
     #[pyo3(get, set)]
     pub keep_all_cells: bool,
     pub overall_traversal: Option<OpaqueIterativeMatcherVal>,
+    #[pyo3(get, set)]
+    pub comment_arg_names: bool,
+    #[pyo3(get, set)]
+    pub commenters: Vec<CircuitCommenter>,
 }
 
 impl Default for PrintHtmlOptions {
@@ -65,11 +69,13 @@ impl Default for PrintHtmlOptions {
             shape_only_when_necessary: true,
             traversal: Some(OpaqueIterativeMatcherVal::for_end_depth(3)),
             display_copy_button: true,
-            colorer: Some(PrintHtmlOptions::variant_colorer()),
+            colorer: Some(PrintHtmlOptions::type_colorer()),
             primary_color: "lightgray".to_owned(),
             display_serial_numbers: false,
             keep_all_cells: true,
             overall_traversal: Some(OpaqueIterativeMatcherVal::never()),
+            comment_arg_names: true,
+            commenters: vec![],
         }
     }
 }
@@ -85,11 +91,13 @@ impl PrintHtmlOptions {
             data: &mut HashMap<HashBytes, NodeJsonData>,
             selfy: &PrintHtmlOptions,
             traversal: Option<OpaqueIterativeMatcherVal>,
+            node_comments: Vec<String>,
         ) -> Result<()> {
             if let Some(_) = data.get(&circ.info().hash) {
                 return Ok(());
             }
-            let desc = selfy.repr_line(circ.clone())?;
+            let mut desc = selfy.repr_line(circ.clone())?;
+            write_comment(&mut desc, node_comments, &selfy.commenters, circ.crc())?;
             data.insert(
                 circ.info().hash,
                 NodeJsonData {
@@ -115,13 +123,18 @@ impl PrintHtmlOptions {
                     return Ok(());
                 };
 
-            for (child, new_traversal) in circ.children().zip(new_traversal_per) {
-                recurse(child, data, selfy, new_traversal)?;
+            for (i, (child, new_traversal)) in circ.children().zip(new_traversal_per).enumerate() {
+                let child_specific_commenters = if selfy.comment_arg_names {
+                    get_child_comments(&(**circ), i)
+                } else {
+                    vec![]
+                };
+                recurse(child, data, selfy, new_traversal, child_specific_commenters)?;
             }
             Ok(())
         }
         for circuit in circuits.clone() {
-            recurse(circuit, data, &self, self.overall_traversal.clone())?;
+            recurse(circuit, data, &self, self.overall_traversal.clone(), vec![])?;
         }
         Ok(())
     }
@@ -154,23 +167,7 @@ impl PrintHtmlOptions {
                 desc.push_str(&old_data.desc);
                 old_data.desc = desc; // Overwrite description to have the right nodes_to_expand count
             } else {
-                desc.push_str(&selfy.repr_line(circ.clone())?);
-                data.insert(
-                    circ.info().hash,
-                    NodeJsonData {
-                        name: circ.name_cloned().unwrap_or("".to_owned()),
-                        desc,
-                        color: match &selfy.colorer {
-                            Some(col_fn) => col_fn.call(circ.clone())?,
-                            None => selfy.primary_color.clone(),
-                        },
-                        children: circ
-                            .clone()
-                            .children()
-                            .map(|c| hash_to_string(&c.info().hash))
-                            .collect(),
-                    },
-                );
+                bail!(anyhow!("overall traversal does not cover traversal."))
             }
 
             let new_traversal_per =
@@ -228,7 +225,9 @@ impl PrintHtmlOptions {
         primary_color = "PrintHtmlOptions::default().primary_color",
         display_serial_numbers = "PrintHtmlOptions::default().display_serial_numbers",
         keep_all_cells = "PrintHtmlOptions::default().keep_all_cells",
-        overall_traversal = "PrintHtmlOptions::default().overall_traversal.unwrap()"
+        overall_traversal = "PrintHtmlOptions::default().overall_traversal.unwrap()",
+        comment_arg_names = "PrintOptions::default().comment_arg_names",
+        commenters = "PrintOptions::default().commenters"
     )] // None are mapped to the default
     pub fn py_new(
         shape_only_when_necessary: bool,
@@ -239,6 +238,8 @@ impl PrintHtmlOptions {
         display_serial_numbers: bool,
         keep_all_cells: bool,
         overall_traversal: Option<OpaqueIterativeMatcherVal>,
+        comment_arg_names: bool,
+        commenters: Vec<CircuitCommenter>,
     ) -> Result<Self> {
         let result = Self {
             shape_only_when_necessary,
@@ -249,6 +250,8 @@ impl PrintHtmlOptions {
             display_serial_numbers,
             keep_all_cells,
             overall_traversal,
+            comment_arg_names,
+            commenters,
         };
         Ok(result)
     }
@@ -261,7 +264,9 @@ impl PrintHtmlOptions {
         display_copy_button = "Default::default()",
         display_serial_numbers = "Default::default()",
         keep_all_cells = "Default::default()",
-        overall_traversal = "Default::default()"
+        overall_traversal = "Default::default()",
+        comment_arg_names = "Default::default()",
+        commenters = "Default::default()"
     )]
     pub fn evolve(
         &self,
@@ -273,6 +278,8 @@ impl PrintHtmlOptions {
         display_serial_numbers: MaybeNotSet<bool>,
         keep_all_cells: MaybeNotSet<bool>,
         overall_traversal: MaybeNotSet<Option<OpaqueIterativeMatcherVal>>,
+        comment_arg_names: MaybeNotSet<bool>,
+        commenters: MaybeNotSet<Vec<CircuitCommenter>>,
     ) -> Self {
         let cloned = self.clone();
         Self {
@@ -288,6 +295,8 @@ impl PrintHtmlOptions {
                 .unwrap_or(cloned.display_serial_numbers),
             keep_all_cells: keep_all_cells.0.unwrap_or(cloned.keep_all_cells),
             overall_traversal: overall_traversal.0.unwrap_or(cloned.overall_traversal),
+            comment_arg_names: comment_arg_names.0.unwrap_or(cloned.comment_arg_names),
+            commenters: commenters.0.unwrap_or(cloned.commenters),
         }
     }
 
@@ -437,6 +446,8 @@ impl PrintHtmlOptions {
             MaybeNotSet(None),
             MaybeNotSet(None),
             MaybeNotSet(None),
+            MaybeNotSet(None),
+            MaybeNotSet(None),
         )
     }
 
@@ -454,12 +465,14 @@ impl PrintHtmlOptions {
     #[staticmethod]
     pub fn size_colorer() -> CircuitToColor {
         CircuitToColor::new_dyn(Box::new(|c| {
-            Ok(color_from_colorcode(circuit_to_size_color_code(c)))
+            Ok(color_from_colorcode(
+                circuit_to_size_color_code(c).unwrap_or(0), // TODO: fix me
+            ))
         }))
     }
 
     #[staticmethod]
-    pub fn variant_colorer() -> CircuitToColor {
+    pub fn type_colorer() -> CircuitToColor {
         CircuitToColor::new_dyn(Box::new(|circuit: CircuitRc| {
             let mut hasher = std::collections::hash_map::DefaultHasher::default();
             circuit.variant_string().hash(&mut hasher);
@@ -481,7 +494,9 @@ impl PrintHtmlOptions {
     #[staticmethod]
     pub fn computability_colorer() -> CircuitToColor {
         CircuitToColor::new_dyn(Box::new(|c| {
-            Ok(color_from_colorcode(circuit_to_computability_color_code(c)))
+            Ok(color_from_colorcode(
+                circuit_to_computability_color_code(c).unwrap_or(0), // TODO: fix me
+            ))
         }))
     }
 }
