@@ -24,7 +24,7 @@ pub fn strip_names(circuit: CircuitRc, context: &mut OptimizationContext) -> Cir
 
 pub fn remove_autotags(circuit: CircuitRc) -> CircuitRc {
     deep_map_op(circuit.clone(), |c| {
-        if let Some(at) = c.as_auto_tag() {
+        if let Some(at) = c.as_tag() {
             return Some(at.node.clone());
         }
         None
