@@ -1,8 +1,6 @@
 // temporary schedule sending setup
 use anyhow::Result;
-use circuit_base::{
-    parsing::parse_circuit, print::PrintOptions, CircuitNode, IrreducibleNode, ScalarConstant,
-};
+use circuit_base::{parsing::Parser, print::PrintOptions, CircuitNode, IrreducibleNode, Scalar};
 use miniserde::{json, Deserialize, Serialize};
 use pyo3::{prelude::*, types::PyByteArray};
 use rr_util::{
@@ -29,7 +27,7 @@ pub struct InstructionToSend {
 pub struct ScheduleToSend {
     pub instructions: Vec<InstructionToSend>,
     pub constants: ::std::collections::HashMap<usize, String>,
-    pub scalar_constants: ::std::collections::HashMap<usize, String>,
+    pub scalars: ::std::collections::HashMap<usize, String>,
     pub dtype: String,
     pub output_circuit: (usize, Vec<usize>),
     pub split_shapes: Option<Vec<Vec<usize>>>,
@@ -82,12 +80,12 @@ impl ScheduleToSend {
         let mut result = Schedule {
             instructions: vec![],
             constants: Default::default(),
-            scalar_constants: Default::default(),
+            scalars: Default::default(),
             device_dtype: TorchDeviceDtype {
                 device,
                 dtype: self.dtype,
             },
-            output_circuit: Some((self.output_circuit.0, ScalarConstant::nrc(0.0, sv![], None))),
+            output_circuit: Some((self.output_circuit.0, Scalar::nrc(0.0, sv![], None))),
             split_shapes: self
                 .split_shapes
                 .map(|z| z.iter().map(|z| z.iter().cloned().collect()).collect()),
@@ -103,40 +101,28 @@ impl ScheduleToSend {
                 .collect(),
         };
 
+        let parser = Parser::default();
         let mut shapes: HashMap<usize, Shape> = <HashMap<usize, Shape> as Default>::default();
         result.constants = self
             .constants
             .iter()
             .map(|(k, v)| {
-                let parsed = parse_circuit(
-                    &("0".to_owned() + v),
-                    Default::default(),
-                    Default::default(),
-                    false,
-                    result.device_dtype.clone().into(),
-                    cache,
-                )?;
+                let parsed = parser.parse_circuit(&v, cache)?;
                 shapes.insert(*k, parsed.info().shape.clone());
                 let irreducible: Option<IrreducibleNode> = (**parsed).clone().into();
                 Ok((*k, irreducible.unwrap()))
             })
             .collect::<Result<_>>()?;
-        result.scalar_constants = self
-            .scalar_constants
+        result.scalars = self
+            .scalars
             .iter()
             .map(|(k, v)| {
                 Ok((*k, {
-                    let resulty = parse_circuit(
-                        &("0".to_owned() + v),
-                        Default::default(),
-                        Default::default(),
-                        false,
-                        result.device_dtype.clone().into(),
-                        cache,
-                    )?
-                    .as_scalar_constant()
-                    .unwrap()
-                    .clone();
+                    let resulty = parser
+                        .parse_circuit(&v, cache)?
+                        .as_scalar()
+                        .unwrap()
+                        .clone();
                     shapes.insert(*k, resulty.info().shape.clone());
                     resulty
                 }))
@@ -150,19 +136,7 @@ impl ScheduleToSend {
                 match v {
                     "Drop" => Ok(Instruction::Drop(ins.key)),
                     "Compute" => Ok(Instruction::Compute(ins.key, {
-                        let result = parse_circuit(
-                            &std::iter::once("0".to_owned() + &ins.info)
-                                .chain(ins.children.iter().map(|i| {
-                                    format!("  {} '{}' {:?} Symbol", *i + 1, i, shapes[i])
-                                }))
-                                .collect::<Vec<String>>()
-                                .join("\n"),
-                            Default::default(),
-                            Default::default(),
-                            false,
-                            Default::default(),
-                            cache,
-                        )?;
+                        let result = parser.parse_circuit(&ins.info, cache)?;
                         shapes.insert(ins.key, result.info().shape.clone());
                         result
                     })),
@@ -181,7 +155,12 @@ impl TryFrom<&Schedule> for ScheduleToSend {
 
     fn try_from(x: &Schedule) -> Result<Self, Self::Error> {
         fn options() -> PrintOptions {
-            PrintOptions::new(true, false, None, false, false).unwrap()
+            PrintOptions {
+                shape_only_when_necessary: false,
+                ..PrintOptions::default()
+            }
+            .validate_ret()
+            .unwrap()
         }
 
         fn map_to_str<N: CircuitNode + Clone>(
@@ -218,7 +197,7 @@ impl TryFrom<&Schedule> for ScheduleToSend {
                 })
                 .collect::<Result<_>>()?,
             constants: map_to_str(&x.constants)?,
-            scalar_constants: map_to_str(&x.scalar_constants)?,
+            scalars: map_to_str(&x.scalars)?,
             dtype: x.device_dtype.dtype.clone(),
             output_circuit: (
                 x.output_circuit.clone().unwrap().0,
