@@ -1,4 +1,4 @@
-use std::iter::zip;
+use std::{fmt::Write, iter::zip};
 
 use anyhow::Result;
 use circuit_base::{
@@ -9,16 +9,25 @@ use pyo3::prelude::*;
 use rr_util::util::{indent, HashBytes};
 use uuid::uuid;
 
-#[pyfunction(hash_child_count = "true", hash_child_shapes = "false")]
+#[pyfunction(
+    hash_child_count = "true",
+    hash_child_shapes = "false",
+    hash_name = "true"
+)]
 pub fn compute_self_hash(
     circuit: CircuitRc,
     hash_child_count: bool,
     hash_child_shapes: bool,
+    hash_name: bool,
 ) -> HashBytes {
     let mut m = blake3::Hasher::new();
     for l in &circuit.info().shape {
         m.update(&l.to_le_bytes());
     }
+    m.update(uuid!("92814fb9-3aa0-489c-b016-0f936941535b").as_bytes());
+    if hash_name {
+        m.update(circuit.name().unwrap_or("").as_bytes());
+    }
     m.update(uuid!("6261daa8-0085-46f7-9f38-b085601fa628").as_bytes());
     if hash_child_count {
         m.update(&circuit.children().count().to_le_bytes());
@@ -37,17 +46,26 @@ pub fn compute_self_hash(
     m.finalize().into()
 }
 
+const SAME_SELF_COLOR: Option<usize> = Some(3);
+const SAME_COLOR: Option<usize> = None;
+const NEW_COLOR: Option<usize> = Some(1);
+const REMOVED_COLOR: Option<usize> = Some(0);
+
 #[pyfunction(
     options = "Default::default()",
-    hash_child_count = "true",
-    hash_child_shapes = "false"
+    require_child_count_same = "true",
+    require_child_shapes_same = "false",
+    require_name_same = "true",
+    print_legend = "true"
 )]
 pub fn diff_circuits(
     new: CircuitRc,
     old: CircuitRc,
     options: PrintOptions,
-    hash_child_count: bool,
-    hash_child_shapes: bool,
+    require_child_count_same: bool,
+    require_child_shapes_same: bool,
+    require_name_same: bool,
+    print_legend: bool,
 ) -> Result<String> {
     let mut options = options;
     options.bijection = false;
@@ -58,13 +76,10 @@ pub fn diff_circuits(
         result: &mut String,
         options: &PrintOptions,
         last_child_stack: Vec<bool>,
-        hash_child_count: bool,
-        hash_child_shapes: bool,
+        require_child_count_same: bool,
+        require_child_shapes_same: bool,
+        require_name_same: bool,
     ) -> Result<()> {
-        const SAME_SELF_COLOR: usize = 3;
-        const SAME_COLOR: usize = 4; // TODO: this is just a random color atm
-        const NEW_COLOR: usize = 1;
-        const REMOVED_COLOR: usize = 0;
         if new == old {
             let mut new_options = options.clone();
             new_options.colorer = Some(PrintOptions::fixed_color(SAME_COLOR));
@@ -75,9 +90,17 @@ pub fn diff_circuits(
             result.push_str("\n");
             return Ok(());
         }
-        if compute_self_hash(new.clone(), hash_child_count, hash_child_shapes)
-            == compute_self_hash(old.clone(), hash_child_count, hash_child_shapes)
-        {
+        if compute_self_hash(
+            new.clone(),
+            require_child_count_same,
+            require_child_shapes_same,
+            require_name_same,
+        ) == compute_self_hash(
+            old.clone(),
+            require_child_count_same,
+            require_child_shapes_same,
+            require_name_same,
+        ) {
             let line_prefix = if let Some(name) = new.name() {
                 name.to_owned() + " "
             } else {
@@ -106,8 +129,9 @@ pub fn diff_circuits(
                     result,
                     options,
                     new_child_stack,
-                    hash_child_count,
-                    hash_child_shapes,
+                    require_child_count_same,
+                    require_child_shapes_same,
+                    require_name_same,
                 )?;
             }
             return Ok(());
@@ -136,8 +160,20 @@ pub fn diff_circuits(
         &mut result,
         &options,
         vec![],
-        hash_child_count,
-        hash_child_shapes,
+        require_child_count_same,
+        require_child_shapes_same,
+        require_name_same,
     )?;
+    if print_legend {
+        write!(
+            result,
+            "{} {} {} {}\n",
+            color("Black: Same as original", None),
+            color("Blue: Children changed", SAME_SELF_COLOR),
+            color("Green: New", NEW_COLOR),
+            color("Red: Removed", REMOVED_COLOR)
+        )
+        .unwrap();
+    }
     Ok(result)
 }
