@@ -3,7 +3,7 @@ import functools
 import itertools
 import torch
 import torch.nn.functional
-from typing import List, Tuple
+from typing import List, Optional, Tuple
 import math
 import itertools
 
@@ -148,8 +148,12 @@ def tensor_from_bytes(device_dtype, shape, bytes, count):
 einsum = lambda tensors_and_axes, out_axes: torch.einsum(*itertools.chain(*tensors_and_axes), out_axes)
 
 
-def random_indices(input: torch.Tensor, shape, replacement: bool) -> torch.Tensor:
+def random_indices(input: torch.Tensor, shape, replacement: bool, seed: Optional[int] = None) -> torch.Tensor:
     shape = tuple(shape)
+
+    if seed is not None:
+        torch.manual_seed(seed)
+
     return torch.multinomial(
         input,
         num_samples=math.prod(shape),
