@@ -22,7 +22,7 @@ use uuid::uuid;
 
 use super::py_types::ExtraPySelfOps;
 use crate::{
-    filter_by_variant,
+    atr, filter_by_variant,
     lru_cache::TensorCacheRrfs,
     py_types::{Tensor, PY_UTILS},
     pycall, python_error_exception, sv,
@@ -652,7 +652,7 @@ impl TensorIndex {
         hasher.finalize().into()
     }
 
-    pub fn repr_bijection(&self) -> Result<String> {
+    pub fn repr_bijection(&self, is_full_text: bool) -> Result<String> {
         let mut result = "[".to_owned();
         for (i, idx) in self.0.iter().enumerate() {
             result.push_str(&match idx {
@@ -663,8 +663,19 @@ impl TensorIndex {
                         + &slice.stop.map(|i| i.to_string()).unwrap_or("".to_owned())
                 }
                 TensorAxisIndex::Tensor(tensor) => {
-                    save_tensor((**tensor).clone(), false)?;
-                    "t".to_owned() + &base16::encode_lower(&tensor.hash().unwrap())[..10]
+                    if is_full_text {
+                        pycall!(
+                            atr!(
+                                pycall!(atr!(tensor.tensor(), tolist, raw), (), raw),
+                                __repr__,
+                                raw
+                            ),
+                            ()
+                        )
+                    } else {
+                        save_tensor((**tensor).clone(), false)?;
+                        "t".to_owned() + &base16::encode_lower(&tensor.hash().unwrap())[..10]
+                    }
                 }
             });
             if i != self.0.len() - 1 {
@@ -674,6 +685,7 @@ impl TensorIndex {
         result.push(']');
         Ok(result)
     }
+
     pub fn from_bijection_string(
         string: &str,
         tensor_cache: &mut Option<TensorCacheRrfs>,
@@ -823,6 +835,10 @@ pub fn compose(top: &TensorIndex, bottom: &TensorIndex) -> Option<TensorIndex> {
     Some(TensorIndex(result))
 }
 
+pub fn string_escape(s: &String) -> String {
+    format!("\"{}\"", s.replace('\\', r"\\").replace('"', "\\\""))
+}
+
 /// use string so you can compare fast when constructing and have full range (weird types like uint or bfloat16)
 /// as opposed to PyObject torch.dtype or enum (could switch to enum)
 #[pyclass]
@@ -847,6 +863,16 @@ impl TorchDeviceDtypeOp {
     fn new(device: Option<String>, dtype: Option<String>) -> Self {
         Self { device, dtype }
     }
+
+    pub fn __repr__(&self) -> String {
+        format!(
+            "TorchDeviceDtypeOp(device={}, dtype={})",
+            self.device
+                .as_ref()
+                .map_or("None".to_owned(), string_escape),
+            self.dtype.as_ref().map_or("None".to_owned(), string_escape)
+        )
+    }
 }
 
 #[pyclass]
@@ -880,6 +906,18 @@ impl TorchDeviceDtype {
                 .unwrap()
         })
     }
+
+    pub fn op(&self) -> TorchDeviceDtypeOp {
+        self.clone().into()
+    }
+
+    pub fn __repr__(&self) -> String {
+        format!(
+            "TorchDeviceDtype(device={}, dtype={})",
+            string_escape(&self.device),
+            string_escape(&self.dtype)
+        )
+    }
 }
 
 impl TorchDeviceDtype {
@@ -971,6 +1009,20 @@ impl TorchDeviceDtypeOp {
     pub fn size(&self) -> usize {
         self.clone().unwrap_or_defaults().size()
     }
+    pub fn overwrite(&self, concrete: &TorchDeviceDtype) -> TorchDeviceDtype {
+        let mut result = concrete.clone();
+        if let Some(x) = &self.device {
+            result.device = x.clone();
+        }
+        if let Some(x) = &self.dtype {
+            result.dtype = x.clone();
+        }
+        result
+    }
+    pub fn cast_tensor(&self, tensor: Tensor) -> Tensor {
+        let concrete = self.overwrite(&TorchDeviceDtype::from_tensor(&tensor));
+        concrete.cast_tensor(tensor)
+    }
 }
 
 impl Default for TorchDeviceDtype {
