@@ -1,7 +1,6 @@
 use circuit_base::{
     expand_node::ExpandError,
     generalfunction::GeneralFunctionShapeError,
-    opaque_iterative_matcher::IncorrectOpaqueTypeError,
     parsing::{ParseArgError, ParseCircuitError},
     ConstructError, TensorEvalError,
 };
@@ -66,7 +65,6 @@ setup_errors!(
     PushDownIndexError,
     DistributeError,
     SimpConfigError,
-    IncorrectOpaqueTypeError,
     GeneralFunctionShapeError,
     ParseArgError,
     SymbolicSizeOverflowError,
